@@ -25,6 +25,9 @@ try:
 except ImportError:
     np = None
 
+USER_CONFIG_FILE = os.path.join(os.path.expanduser("~"), 'Library', 'Containers',
+                                'com.microsoft.Excel', 'Data', 'xlwings.conf')
+
 # Time types
 time_types = (dt.date, dt.datetime)
 if np:
@@ -131,6 +134,10 @@ class App:
     def display_alerts(self, value):
         self.xl.display_alerts.set(value)
 
+    @property
+    def startup_path(self):
+        return hfs_to_posix_path(self.xl.startup_path.get())
+
     @property
     def calculation(self):
         return calculation_k2s[self.xl.calculation.get()]
@@ -157,6 +164,14 @@ class App:
         kwargs = {'arg{0}'.format(i): n for i, n in enumerate(args, 1)}
         return self.xl.run_VB_macro(macro, **kwargs)
 
+    @property
+    def status_bar(self):
+        return self.xl.status_bar.get()
+
+    @status_bar.setter
+    def status_bar(self, value):
+        self.xl.status_bar.set(value)
+
 
 class Books:
 
@@ -593,6 +608,11 @@ class Range:
     def top(self):
         return self.xl.properties().get(kw.top)
 
+    @property
+    def has_array(self):
+        if self.xl is not None:
+            return self.xl.has_array.get()
+
     @property
     def number_format(self):
         if self.xl is not None:
@@ -769,6 +789,7 @@ class Range:
     def unmerge(self):
         self.xl.unmerge()
 
+
 class Shape:
     def __init__(self, parent, key):
         self.parent = parent
@@ -833,6 +854,14 @@ class Shape:
         # self.xl.activate_object()  # doesn't work?
         self.xl.select()
 
+    def scale_height(self, factor, relative_to_original_size, scale):
+        self.xl.scale_height(scale=scaling[scale], relative_to_original_size=relative_to_original_size,
+                             factor=factor)
+
+    def scale_width(self, factor, relative_to_original_size, scale):
+        self.xl.scale_width(scale=scaling[scale], relative_to_original_size=relative_to_original_size,
+                            factor=factor)
+
 
 class Collection:
 
@@ -1368,4 +1397,10 @@ shape_types_k2s = {
     kw.shape_type_ink_comment: 'ink_comment',
 }
 
+scaling = {
+    "scale_from_top_left": kw.scale_from_top_left,
+    "scale_from_bottom_right": kw.scale_from_bottom_right,
+    "scale_from_middle": kw.scale_from_middle
+}
+
 shape_types_s2k = {v: k for k, v in shape_types_k2s.items()}
