@@ -325,6 +325,16 @@ class App:
     def display_alerts(self, value):
         self.impl.display_alerts = value
 
+    @property
+    def startup_path(self):
+        """
+        Returns the path to ``XLSTART`` which is where the xlwings add-in gets
+        copied to by doing ``xlwings addin install``.
+
+        .. versionadded:: 0.19.4
+        """
+        return self.impl.startup_path
+
     @property
     def calculation(self):
         """
@@ -389,18 +399,6 @@ class App:
         """
         return Range(impl=self.impl.range(cell1, cell2))
 
-    def __repr__(self):
-        return "<Excel App %s>" % self.pid
-
-    def __eq__(self, other):
-        return type(other) is App and other.pid == self.pid
-
-    def __ne__(self, other):
-        return not self.__eq__(other)
-
-    def __hash__(self):
-        return hash(self.pid)
-
     def macro(self, name):
         """
         Runs a Sub or Function in Excel VBA that are not part of a specific workbook but e.g. are part of an add-in.
@@ -433,6 +431,31 @@ class App:
         """
         return Macro(self, name)
 
+    @property
+    def status_bar(self):
+        """
+        Gets or sets the value of the status bar. Returns ``False`` if Excel has control of it.
+
+        .. versionadded:: 0.20.0
+        """
+        return self.impl.status_bar
+
+    @status_bar.setter
+    def status_bar(self, value):
+        self.impl.status_bar = value
+
+    def __repr__(self):
+        return "<Excel App %s>" % self.pid
+
+    def __eq__(self, other):
+        return type(other) is App and other.pid == self.pid
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+    def __hash__(self):
+        return hash(self.pid)
+
 
 class Book:
     """
@@ -1003,11 +1026,15 @@ class Range:
             else:
                 raise ValueError("Invalid arguments")
 
-        self.impl = impl
+        self._impl = impl
 
         # Keyword Arguments
         self._options = options
 
+    @property
+    def impl(self):
+        return self._impl
+
     @property
     def api(self):
         """
@@ -1152,6 +1179,13 @@ class Range:
         """Clears the content and the formatting of a Range."""
         return self.impl.clear()
 
+    @property
+    def has_array(self):
+        """
+        Are we part of an Array formula?
+        """
+        return self.impl.has_array
+
     def end(self, direction):
         """
         Returns a Range object that represents the cell at the end of the region that contains the source range.
@@ -2866,6 +2900,7 @@ def view(obj, sheet=None):
 
     sheet.range('A1').value = obj
     sheet.autofit()
+    sheet.book.app.activate(steal_focus=True)
 
 
 class Macro:
